package game;

import java.awt.Point;
import java.util.ArrayList;

public class State {
  private static final int DRONE_WEIGHT = 100;
  private static final int QUEEN_WEIGHT = 500;
  private static final double DANGER_MULT = 0.3;
  private static final double CLUSTERING_THRESHOLD = 0.2;
  private static final int CLUSTER_WEIGHT = 2;
  
//  private int heuristic = 0;
  private Player activePlayer;
  private ArrayList <Piece> pieces;
  private ArrayList <Player> participants;
  private boolean callNext = false;
//  private ArrayList <Move> moves = new ArrayList <Move> (); // Listing of moves to get to this state
  // TODO: Record moves to provide complete list of actions
  
  private State(Bullies game, Player evaluator) {
//    this.heuristic = evaluate(game, evaluator);
    this.activePlayer = game.getActivePlayer();
    
    this.pieces = new ArrayList <Piece> ();
    this.participants = game.getParticipants();
    for (Piece piece : game.getPieces()) {
      this.pieces.add(new Piece(piece));
    }
  }
  
  public static State push(Bullies game, Player evaluator) {
    return new State(game, evaluator);
  }
  
  public static void pop(Bullies game, State state) {
    ArrayList <Piece> restoringPieces = new ArrayList <Piece> ();
    for (Piece piece : state.pieces) {
      restoringPieces.add(new Piece(piece));
    }
    game.setState(restoringPieces, state.activePlayer, state.participants);
    if (state.callNext) game.nextTurn();
  }
  
  public void setCallNext(boolean callNext) {
    this.callNext = callNext;
  }
  
  public static int evaluate(Bullies game, Player evaluator) {
    // TODO: Might be helped by checking for blockers and queen's distance from enemy drones
    int heuristic = 0;
    int maxShove = Math.max(Piece.Type.DRONE.getShoveDistance(), Piece.Type.QUEEN.getShoveDistance());
    int dangerousX = game.getBoard().getWidth() - maxShove - 1;
    int dangerousY = game.getBoard().getHeight() - maxShove - 1;
    ArrayList <Piece> pieces = game.getPieces();
    Point evaluatorCenter = new Point(0, 0);
    int evaluatorCount = 0;
    Point opponentCenter = new Point(0, 0);
    int opponentCount = 0;
    int clusterThreshold = (int) ((game.getBoard().getWidth() + game.getBoard().getHeight()) * CLUSTERING_THRESHOLD);
    
    for (Piece temp : pieces) {
      // Piece count
      int sign = temp.getOwner().equals(evaluator) ? 1 : -1;
      int magnitude = temp.getType() == Piece.Type.DRONE ? DRONE_WEIGHT : QUEEN_WEIGHT;
      heuristic += sign * magnitude;
      
      // Checks if within shove distance of edge
      int dangerDistance = 0;
      dangerDistance = Math.max(dangerDistance, maxShove - temp.getAnchor().x);
      dangerDistance = Math.max(dangerDistance, temp.getAnchor().x - dangerousX);
      dangerDistance = Math.max(dangerDistance, maxShove - temp.getAnchor().y);
      dangerDistance = Math.max(dangerDistance, temp.getAnchor().y - dangerousY);
      if (dangerDistance > 0) {
        heuristic += -DANGER_MULT * sign * magnitude * dangerDistance;
      }
      
      /*
      boolean dangerousLocation = false;
      dangerousLocation |= temp.getAnchor().x < maxShove;
      dangerousLocation |= temp.getAnchor().x > dangerousX;
      dangerousLocation |= temp.getAnchor().y < maxShove;
      dangerousLocation |= temp.getAnchor().y > dangerousY;
      if (dangerousLocation) {
        heuristic += -DANGER_MULT * sign * magnitude;
      }
      */
//      /*
      // Adjusts center of cloud and checks deviation (promoting clustering)
      if (temp.getOwner().equals(evaluator)) {
        ++evaluatorCount;
        evaluatorCenter.x += temp.getAnchor().x;
        evaluatorCenter.y += temp.getAnchor().y;
        if ((evaluatorCenter.x + evaluatorCenter.y) / evaluatorCount > clusterThreshold) {
          heuristic += -CLUSTER_WEIGHT;
        }
      } else {
        ++opponentCount;
        opponentCenter.x += temp.getAnchor().x;
        opponentCenter.y += temp.getAnchor().y;
        if ((opponentCenter.x + opponentCenter.y) / opponentCount > clusterThreshold) {
          heuristic += CLUSTER_WEIGHT;
        }
      }
//      */
    }
    
//    /*
    if (evaluatorCount == 0) return Integer.MIN_VALUE;
    else if (opponentCount == 0) return Integer.MAX_VALUE;
    
    // Checks cluster distances from center
    int centerX = game.getBoard().getWidth() / 2;
    int centerY = game.getBoard().getHeight() / 2;
    heuristic -= (evaluatorCenter.x / evaluatorCount) - centerX;
    heuristic -= (evaluatorCenter.y / evaluatorCount) - centerY;
    heuristic += (opponentCenter.x / opponentCount) - centerX;
    heuristic += (opponentCenter.y / opponentCount) - centerY;
//    */
    return heuristic;
  }
  
  /*
  public class Move {
    public Point location;
    public Direction dir;
    
    public Move(Point location, Direction dir) {
      this.location = location;
      this.dir = dir;
    }
  }
  */
  
  public boolean equals(Object o) {
    if (this == o) return true;
    else if (o instanceof State) {
      State s = (State) o;
      boolean equals = true;
      for (Piece piece : this.pieces) {
        equals &= s.pieces.contains(piece);
      }
      
      return equals;
    } else return false;
  }
}
