package util;

import java.awt.Point;

/**
 * Representation of the four corners in a rectangular figure.
 * @author Damian Johnson (atagar1@gmail.com)
 * @version April 25, 2007
 */
public enum Corner implements Orientation {
  UPPER_LEFT, UPPER_RIGHT, LOWER_RIGHT, LOWER_LEFT;
  
  /**
   * Returns the enum representation of a string. This is case sensitive.
   * @param str toString representation of this enum
   * @return enum associated with a string
   * @throws IllegalArgumentException if argument is not represented by this enum.
   */
  public static Corner fromString(String str) {
    for (Corner type : Corner.values()) {
      if (str.equals(type.toString())) return type;
    }
    throw new IllegalArgumentException();
  }
  
  /**
   * Returns a random corner.
   * @return randomly generated corner
   */
  public static Corner random() {
    int random = (int) (Math.random() * 4);
    
    if (random == 0) return UPPER_LEFT;
    else if (random == 1) return UPPER_RIGHT;
    else if (random == 2) return LOWER_RIGHT;
    else if (random == 3) return LOWER_LEFT;
    else {
      assert false : "Faulty bounds on random value: " + random;
      return null;
    }
  }
  
  public Corner right() {
    if (this == UPPER_LEFT) return UPPER_RIGHT;
    else if (this == UPPER_RIGHT) return LOWER_RIGHT;
    else if (this == LOWER_RIGHT) return LOWER_LEFT;
    else if (this == LOWER_LEFT) return UPPER_LEFT;
    else {
      assert false : this;
      return null;
    }
  }
  
  public Corner left() {
    if (this == UPPER_LEFT) return LOWER_LEFT;
    else if (this == UPPER_RIGHT) return UPPER_LEFT;
    else if (this == LOWER_RIGHT) return UPPER_RIGHT;
    else if (this == LOWER_LEFT) return LOWER_RIGHT;
    else {
      assert false : this;
      return null;
    }
  }
  
  public Corner reverse() {
    if (this == UPPER_LEFT) return LOWER_RIGHT;
    else if (this == UPPER_RIGHT) return LOWER_LEFT;
    else if (this == LOWER_RIGHT) return UPPER_LEFT;
    else if (this == LOWER_LEFT) return UPPER_RIGHT;
    else {
      assert false : this;
      return null;
    }
  }
  
  public Point getOffset(Point loc) {
    if (this == UPPER_LEFT) return new Point(loc.x - 1, loc.y - 1);
    else if (this == UPPER_RIGHT) return new Point(loc.x + 1, loc.y - 1);
    else if (this == LOWER_RIGHT) return new Point(loc.x + 1, loc.y + 1);
    else if (this == LOWER_LEFT) return new Point(loc.x - 1, loc.y + 1);
    else {
      assert false : this;
      return null;
    }
  }
  
  /**
   * Returns if the corner is upper or lower.
   * @return true if enum is upper, false otherwise
   */
  public boolean isUpper() {
    return this == UPPER_LEFT || this == UPPER_RIGHT;
  }
  
  /**
   * Returns if the corner is left or right.
   * @return true if enum is left, false otherwise
   */
  public boolean isLeft() {
    return this == UPPER_LEFT || this == LOWER_LEFT;
  }
  
  @Override
  public String toString() {
    return StringTools.getReadableConstant(this.name());
  }
}
