package util;

import java.awt.Point;

/**
 * Representation of the four directional orientations: up, right, down, and left.
 * @author Damian Johnson (atagar1@gmail.com) and Marty Stepp
 * @version April 25, 2007
 */
public enum Direction implements Orientation {
  UP, RIGHT, DOWN, LEFT;
  
  /**
   * Returns the enum representation of a string. This is case sensitive.
   * @param str toString representation of this enum
   * @return enum associated with a string
   * @throws IllegalArgumentException if argument is not represented by this enum.
   */
  public static Direction fromString(String str) {
    for (Direction type : Direction.values()) {
      if (str.equals(type.toString())) return type;
    }
    throw new IllegalArgumentException();
  }
  
  /**
   * Returns a random direction.
   * @return randomly generated direction
   */
  public static Direction random() {
    int random = (int) (Math.random() * 4);
    
    if (random == 0) return UP;
    else if (random == 1) return RIGHT;
    else if (random == 2) return DOWN;
    else if (random == 3) return LEFT;
    else {
      assert false : "Faulty bounds on random value: " + random;
      return null;
    }
  }
  
  public Direction right() {
    if (this == UP) return RIGHT;
    else if (this == RIGHT) return DOWN;
    else if (this == DOWN) return LEFT;
    else if (this == LEFT) return UP;
    else {
      assert false : this;
      return null;
    }
  }
  
  public Direction left() {
    if (this == UP) return LEFT;
    else if (this == RIGHT) return UP;
    else if (this == DOWN) return RIGHT;
    else if (this == LEFT) return DOWN;
    else {
      assert false : this;
      return null;
    }
  }
  
  public Direction reverse() {
    if (this == UP) return DOWN;
    else if (this == RIGHT) return LEFT;
    else if (this == DOWN) return UP;
    else if (this == LEFT) return RIGHT;
    else {
      assert false : this;
      return null;
    }
  }
  
  public Point getOffset(Point loc) {
    if (this == UP) return new Point(loc.x, loc.y - 1);
    else if (this == RIGHT) return new Point(loc.x + 1, loc.y);
    else if (this == DOWN) return new Point(loc.x, loc.y + 1);
    else if (this == LEFT) return new Point(loc.x - 1, loc.y);
    else {
      assert false : this;
      return null;
    }
  }
  
  /**
   * Returns if the direction concerns vertical or horizontal directions.
   * @return true if enum is up or down, false otherwise
   */
  public boolean isVertical() {
    return this == Direction.UP || this == Direction.DOWN;
  }
  
  @Override
  public String toString() {
    return StringTools.getReadableConstant(this.name());
  }
}
