package util;

import java.awt.Point;

/**
 * Representation of directional orientations. Known implementing enums: Direction and Corner
 * @author Damian Johnson (atagar1@gmail.com)
 * @version April 25, 2007
 */
public interface Orientation {
  /**
   * Returns the 90 degree clockwise rotation.
   * @return next clockwise orientation
   */
  public abstract Orientation right();
  
  /**
   * Returns the 90 degree counter-clockwise rotation.
   * @return next counter-clockwise orientation
   */
  public abstract Orientation left();
  
  /**
   * Returns the opposite facing direction.
   * @return opposing direction
   */
  public abstract Orientation reverse();
  
  /**
   * Returns a point offset in this direction according to inverted cartesian coordinates (the same
   * as Graphics objects). In other words coordinates grow toward the lower-right, with shifts as
   * follows: Up = -Y Down = +Y Left = -X Right = +X
   * @param loc location to be offset
   * @return shifted point
   */
  public abstract Point getOffset(Point loc);
}
