package util.squareGrid;
/** BoundsException class
 * @author Damian Johnson
 * @version August 26, 2006
 */

import java.awt.Point;
import java.util.*;

/** Exception representing positioning outside a board's 
 * dimensions.
 */
public class BoundsException extends PositionException {
    private static final long serialVersionUID = 0;
    private HashSet <Point> outside = new HashSet <Point> ();
    
    /**
     * Exception constructor for positioning outside the bounds 
     * of a board.
     * @param board     board on which the exception takes place
     * @param perpetrator     placeable object illegally placed
     * @param attemptedAnchor     anchor point to which it failed to be placed
     */
	protected BoundsException(Board board, Placeable perpetrator, Point attemptedAnchor) {
		super(board, perpetrator, attemptedAnchor);
		HashSet <Point> offset = perpetrator.getLocations();
		offset = PointTools.shift(offset, attemptedAnchor);
		for (Point loc : offset) {
			if (!board.inBounds(loc)) this.outside.add(loc);
		}
	}
	
    /**
     * Provides a set of points that would lie outside the 
     * board's dimensions.
     * @return attempted positions that be out of bounds
     */
	public HashSet <Point> getInvalidPoints() {
		return this.outside;
	}
}
