package util.squareGrid;
/** CollisionException class
 * @author Damian Johnson
 * @version August 21, 2006
 */

import java.awt.Point;
import java.util.*;

/** Exception representing an overlapping of placeable positions 
 * within a board.
 */
public class CollisionException extends PositionException {
    private static final long serialVersionUID = 0;
    private HashMap <Point, Placeable> overlap = new HashMap <Point, Placeable> ();
    
    /**
     * Exception constructor for overlapping positions.
     * @param board     board on which the exception takes place
     * @param perpetrator     placeable object illegally placed
     * @param attemptedAnchor     anchor point to which it failed to be placed
     */
	protected CollisionException(Board board, Placeable perpetrator, Point attemptedAnchor) {
		super(board, perpetrator, attemptedAnchor);
		HashSet <Point> offset = perpetrator.getLocations();
		offset = PointTools.shift(offset, attemptedAnchor);
		for (Point loc : offset) {
			Placeable occupant = board.getLocation(loc);
			if (occupant != null && !occupant.equals(perpetrator)) {
				this.overlap.put(loc, occupant);
			}
		}
	}
	
    /**
     * Provides a mapping of conflicting points to the 
     * placeable objects there.
     * @return mapping of conflicting points to placeables
     */
	public HashMap <Point, Placeable> getCollisionMap() {
		return this.overlap;
	}
	
    /**
     * Provides set of placeable objects with overlapping positions.
     * @return placeables occupying overlapping positions
     */
	public HashSet <Placeable> getCollidingPlaceables() {
		return new HashSet <Placeable> (this.overlap.values());
	}
	
    /**
     * Set of points that would overlap other placeables in the board.
     * @return placeables occupying overlapping positions
     */
	public HashSet <Point> getCollidingPoints() {
		return new HashSet <Point> (this.overlap.keySet());
	}
}
