package util.squareGrid;
/** PositionException class
 * @author Damian Johnson
 * @version August 19, 2006
 */

import java.awt.Point;

/** Exception representing illegal positions within a board. 
 * BoundsExceptions hold precidence over CollisionExceptions.
 */
public abstract class PositionException extends RuntimeException {
    private static final long serialVersionUID = 0;
    private Board board;
    private Placeable perpetrator;
    private Point attemptedAnchor;
    
    /**
     * Exception constructor for illegal board positioning.
     * @param board     board on which the exception takes place
     * @param perpetrator     placeable object illegally placed
     * @param attemptedAnchor     anchor point to which it failed 
     * to be placed
     */
    protected PositionException(Board board, Placeable perpetrator, Point attemptedAnchor) {
    	this.board = board;
    	this.perpetrator = perpetrator;
    	this.attemptedAnchor = attemptedAnchor;
    }
    
    /**
     * Returns the board on which the exception takes place.
     * @return board with exception
     */
    public Board getBoard() {
    	return this.board;
    }
    
    /**
     * Returns the placeable object which failed to be added or repositioned.
     * @return failed placeable object
     */
	public Placeable getPerpetrator() {
		return this.perpetrator;
	}
    
    /**
     * Anchor point where the placeable object failed to be added or repositioned.
     * @return anchor point of the failure
     */
	public Point getAttemptedAnchor() {
		return this.attemptedAnchor;
	}
}
